import numpy as np
import pygeos
import pytest

point_polygon_testdata = \
    pygeos.points(np.arange(6), np.arange(6)), pygeos.box(2, 2, 4, 4)

point = pygeos.points(2, 2)
line_string = pygeos.linestrings([[0, 0], [1, 0], [1, 1]])
linear_ring = pygeos.linearrings(((0, 0), (0, 1), (1, 1), (1, 0), (0, 0)))
polygon = pygeos.polygons(((0., 0.), (0., 2.), (2., 2.), (2., 0.), (0., 0.)))
multi_point = pygeos.multipoints([[0.0, 0.0], [1.0, 2.0]])
multi_line_string = pygeos.multilinestrings([[[0.0, 0.0], [1.0, 2.0]]])
multi_polygon = pygeos.multipolygons([
        ((0.0, 0.0), (0.0, 1.0), (1.0, 1.0), (1.0, 0.0)),
        ((0.1, 0.1), (0.1, 0.2), (0.2, 0.2), (0.2, 0.1)),
    ])
geometry_collection = pygeos.geometrycollections(
    [pygeos.points(51, -1), pygeos.linestrings([(52, -1), (49, 2)])]
)
point_z = pygeos.points(1.0, 1.0, 1.0)

UNARY_PREDICATES = (
    pygeos.is_empty,
    pygeos.is_simple,
    pygeos.is_ring,
    pygeos.has_z,
    pygeos.is_closed,
    pygeos.is_valid,
)

BINARY_PREDICATES = (
    pygeos.disjoint,
    pygeos.touches,
    pygeos.intersects,
    pygeos.crosses,
    pygeos.within,
    pygeos.contains,
    pygeos.overlaps,
    pygeos.equals,
    pygeos.covers,
    pygeos.covered_by,
)

all_types = (
    point,
    line_string,
    linear_ring,
    polygon,
    multi_point,
    multi_line_string,
    multi_polygon,
    geometry_collection,
)


def box_tpl(x1, y1, x2, y2):
    return (x2, y1), (x2, y2), (x1, y2), (x1, y1), (x2, y1)

# Y_b


def test_has_z():
    actual = pygeos.has_z([point, point_z])
    expected = [False, True]
    np.testing.assert_equal(actual, expected)

# YY_b


def test_disjoint():
    actual = pygeos.disjoint(*point_polygon_testdata)
    expected = [True, True, False, False, False, True]
    np.testing.assert_equal(actual, expected)


def test_touches():
    actual = pygeos.touches(*point_polygon_testdata)
    expected = [False, False, True, False, True, False]
    np.testing.assert_equal(actual, expected)


def test_intersects():
    actual = pygeos.intersects(*point_polygon_testdata)
    expected = [False, False, True, True, True, False]
    np.testing.assert_equal(actual, expected)


def test_within():
    actual = pygeos.within(*point_polygon_testdata)
    expected = [False, False, False, True, False, False]
    np.testing.assert_equal(actual, expected)


def test_contains():
    actual = pygeos.contains(*reversed(point_polygon_testdata))
    expected = [False, False, False, True, False, False]
    np.testing.assert_equal(actual, expected)

# Y_Y


def test_get_centroid():
    actual = pygeos.get_centroid(polygon)
    assert pygeos.equals(actual, pygeos.points(1, 1))


# Yi_Y


def test_get_point_n():
    actual = pygeos.get_point_n(line_string, np.int16(1))
    assert pygeos.equals(actual, pygeos.points(1, 0))


def test_set_srid():
    actual = pygeos.set_srid(point, np.int16(4326))
    assert pygeos.get_srid(point) == 0
    assert pygeos.get_srid(actual) == 4326


# Yd_Y


def test_simplify():
    line = pygeos.linestrings([[0, 0], [0.1, 1], [0, 2]])
    actual = pygeos.simplify(line, [0, 1.])
    assert pygeos.get_num_points(actual).tolist() == [3, 2]

# YY_Y


def test_intersection():
    poly1, poly2 = pygeos.box(0, 0, 10, 10), pygeos.box(5, 5, 20, 20)
    actual = pygeos.intersection(poly1, poly2)
    expected = pygeos.box(5, 5, 10, 10)
    assert pygeos.equals(actual, expected)


def test_union():
    poly1, poly2 = pygeos.box(0, 0, 10, 10), pygeos.box(10, 0, 20, 10)
    actual = pygeos.union(poly1, poly2)
    expected = pygeos.box(0, 0, 20, 10)
    assert pygeos.equals(actual, expected)

# Y_d


def test_area():
    assert pygeos.area(polygon) == 4.

# Y_B


def test_geom_type_id():
    assert pygeos.geom_type_id(all_types).tolist() == list(range(8))

# Y_i


def test_get_num_points():
    assert pygeos.get_num_points(line_string) == 3


# YY_d


def test_distance():
    actual = pygeos.distance(*point_polygon_testdata)
    expected = [2 * 2**0.5, 2**0.5, 0, 0, 0, 2**0.5]
    np.testing.assert_allclose(actual, expected)

# YY_d_2


def test_project():
    line = pygeos.linestrings([[0, 0], [1, 1], [2, 2]])
    points = pygeos.points([1, 3], [0, 3])
    actual = pygeos.project(line, points)
    expected = [0.5 * 2**0.5, 2 * 2**0.5]
    np.testing.assert_allclose(actual, expected)


# specials


def test_buffer():
    radii = np.array([1., 2.])
    actual = pygeos.buffer(point, radii, np.int16(16))
    assert pygeos.area(actual) == pytest.approx(np.pi * radii**2, rel=0.01)


def test_snap():
    line = pygeos.linestrings([[0, 0], [1, 0], [2, 0]])
    points = pygeos.points([0, 1], [1, 0.1])
    actual = pygeos.snap(points, line, 0.5)
    expected = pygeos.points([0, 1], [1, 0])
    assert pygeos.equals(actual, expected).all()


def test_equals_exact():
    point1 = pygeos.points(0, 0)
    point2 = pygeos.points(0, 0.1)
    actual = pygeos.equals_exact(point1, point2, [0.01, 1.])
    expected = [False, True]
    np.testing.assert_equal(actual, expected)


# construction

def test_points_from_coords():
    actual = pygeos.points([[0, 0], [2, 2]])
    assert actual[0].to_wkt() == "POINT (0 0)"
    assert actual[1].to_wkt() == "POINT (2 2)"


def test_points_from_xy():
    actual = pygeos.points(2, [0, 1])
    assert actual[0].to_wkt() == "POINT (2 0)"
    assert actual[1].to_wkt() == "POINT (2 1)"


def test_points_from_xyz():
    actual = pygeos.points(1, 1, [0, 1])
    assert actual[0].to_wkt() == "POINT Z (1 1 0)"
    assert actual[1].to_wkt() == "POINT Z (1 1 1)"


def test_points_invalid_ndim():
    with pytest.raises(pygeos.GEOSException):
        pygeos.points([0, 1, 2, 3])


def test_linestrings_from_coords():
    actual = pygeos.linestrings([[[0, 0], [1, 1]], [[0, 0], [2, 2]]])
    assert actual[0].to_wkt() == "LINESTRING (0 0, 1 1)"
    assert actual[1].to_wkt() == "LINESTRING (0 0, 2 2)"


def test_linestrings_from_xy():
    actual = pygeos.linestrings([0, 1], [2, 3])
    assert actual.to_wkt() == "LINESTRING (0 2, 1 3)"


def test_linestrings_from_xy_broadcast():
    x = [0, 1]  # the same X coordinates for both linestrings
    y = [2, 3], [4, 5]  # each linestring has a different set of Y coordinates
    actual = pygeos.linestrings(x, y)
    assert actual[0].to_wkt() == "LINESTRING (0 2, 1 3)"
    assert actual[1].to_wkt() == "LINESTRING (0 4, 1 5)"


def test_linestrings_from_xyz():
    actual = pygeos.linestrings([0, 1], [2, 3], 0)
    assert actual.to_wkt() == "LINESTRING Z (0 2 0, 1 3 0)"


def test_linearrings():
    actual = pygeos.linearrings(box_tpl(0, 0, 1, 1))
    assert actual.to_wkt() == "LINEARRING (1 0, 1 1, 0 1, 0 0, 1 0)"


def test_linearrings_from_xy():
    actual = pygeos.linearrings([0, 1, 2, 0], [3, 4, 5, 3])
    assert actual.to_wkt() == "LINEARRING (0 3, 1 4, 2 5, 0 3)"


def test_linearrings_unclosed():
    actual = pygeos.linearrings(box_tpl(0, 0, 1, 1)[:-1])
    assert actual.to_wkt() == "LINEARRING (1 0, 1 1, 0 1, 0 0, 1 0)"


def test_polygon_from_linearring():
    actual = pygeos.polygons(pygeos.linearrings(box_tpl(0, 0, 1, 1)))
    assert actual.to_wkt() == "POLYGON ((1 0, 1 1, 0 1, 0 0, 1 0))"


def test_polygons():
    actual = pygeos.polygons(box_tpl(0, 0, 1, 1))
    assert actual.to_wkt() == "POLYGON ((1 0, 1 1, 0 1, 0 0, 1 0))"


def test_polygon_no_hole_list_raises():
    with pytest.raises(ValueError):
        pygeos.polygons(box_tpl(0, 0, 10, 10), box_tpl(1, 1, 2, 2))


def test_polygon_with_1_hole():
    actual = pygeos.polygons(box_tpl(0, 0, 10, 10), [box_tpl(1, 1, 2, 2)])
    assert pygeos.area(actual) == 99.


def test_polygon_with_2_holes():
    actual = pygeos.polygons(
        box_tpl(0, 0, 10, 10),
        [box_tpl(1, 1, 2, 2), box_tpl(3, 3, 4, 4)]
    )
    assert pygeos.area(actual) == 98.


def test_2_polygons_with_same_hole():
    actual = pygeos.polygons(
        [box_tpl(0, 0, 10, 10), box_tpl(0, 0, 5, 5)],
        [box_tpl(1, 1, 2, 2)]
    )
    assert pygeos.area(actual).tolist() == [99., 24.]


def test_2_polygons_with_2_same_holes():
    actual = pygeos.polygons(
        [box_tpl(0, 0, 10, 10), box_tpl(0, 0, 5, 5)],
        [box_tpl(1, 1, 2, 2), box_tpl(3, 3, 4, 4)]
    )
    assert pygeos.area(actual).tolist() == [98., 23.]


def test_2_polygons_with_different_holes():
    actual = pygeos.polygons(
        [box_tpl(0, 0, 10, 10), box_tpl(0, 0, 5, 5)],
        [[box_tpl(1, 1, 3, 3)], [box_tpl(1, 1, 2, 2)]]
    )
    assert pygeos.area(actual).tolist() == [96., 24.]


def test_box():
    actual = pygeos.box(0, 0, 1, 1)
    assert actual.to_wkt() == "POLYGON ((1 0, 1 1, 0 1, 0 0, 1 0))"


def test_box_multiple():
    actual = pygeos.box(0, 0, [1, 2], [1, 2])
    assert actual[0].to_wkt() == "POLYGON ((1 0, 1 1, 0 1, 0 0, 1 0))"
    assert actual[1].to_wkt() == "POLYGON ((2 0, 2 2, 0 2, 0 0, 2 0))"


# NaN / None handling

def test_Y_b_nan():
    for func in UNARY_PREDICATES:
        actual = func(np.array([np.nan, None]))
        if func is pygeos.is_empty:
            assert actual.all()
        else:
            assert (~actual).all()

def test_YY_b_nan():
    for func in BINARY_PREDICATES:
        actual = func(
            np.array([point, np.nan, np.nan, point, None, None]),
            np.array([np.nan, point, np.nan, None, point, None])
        )
        if func is pygeos.disjoint:
            assert actual.all()
        else:
            assert (~actual).all()

def test_Y_Y_nan():
    actual = pygeos.clone(np.array([point, np.nan, None]))
    assert pygeos.equals(actual[0], point)
    assert np.isnan(actual[1])
    assert np.isnan(actual[2])


def test_Y_d_nan():
    actual = pygeos.area(np.array([polygon, np.nan, None]))
    assert actual[0] == pygeos.area(polygon)
    assert np.isnan(actual[1])
    assert np.isnan(actual[2])


def test_YY_Y_nan():
    actual = pygeos.intersection(
        np.array([point, np.nan, np.nan, point, None, None, point]),
        np.array([np.nan, point, np.nan, None, point, None, point])
    )
    assert pygeos.equals(actual[-1], point)
    assert np.isnan(actual[:-1].astype(np.float)).all()


def test_Yd_Y_nan():
    actual = pygeos.simplify(
        np.array([point, np.nan, np.nan, None, point]),
        np.array([np.nan, 1.0, np.nan, 1.0, 1.0])
    )
    assert pygeos.equals(actual[-1], point)
    assert np.isnan(actual[:-1].astype(np.float)).all()


def test_YY_d_nan():
    actual = pygeos.distance(
        np.array([point, np.nan, np.nan, point, None, None, point]),
        np.array([np.nan, point, np.nan, None, point, None, point])
    )
    assert actual[-1] == 0.
    assert np.isnan(actual[:-1].astype(np.float)).all()


def test_create_collection_only_nan():
    actual = pygeos.multipoints(np.array([np.nan], dtype=object))
    assert actual.to_wkt() == 'MULTIPOINT EMPTY'


def test_create_collection_skips_nan():
    actual = pygeos.multipoints([point, np.nan, np.nan, point])
    assert actual.to_wkt() == 'MULTIPOINT (2 2, 2 2)'


# GEOSGeometry


def test_new_from_wkt():
    geom = point
    actual = pygeos.GEOSGeometry(geom.to_wkt())
    assert pygeos.equals(actual, geom)


def test_new_from_wkb():
    geom = point
    actual = pygeos.GEOSGeometry(geom.to_wkb())
    assert pygeos.equals(actual, geom)


def test_new_from_ptr():
    geom = point
    actual = pygeos.GEOSGeometry(geom._ptr)
    assert pygeos.equals(actual, geom)
    assert actual._ptr != geom._ptr


def test_adapt_ptr_raises():
    geom = pygeos.clone(point)
    with pytest.raises(AttributeError):
        geom._ptr += 1


# wkt/wkb io

def test_to_wkt():
    assert point.to_wkt() == "POINT (2 2)"
    assert point.to_wkt(trim=False) == "POINT (2.000000 2.000000)"
    assert point.to_wkt(trim=False, precision=3) == "POINT (2.000 2.000)"
    assert point_z.to_wkt(dimension=2) == "POINT (1 1)"
    assert point_z.to_wkt(dimension=3) == "POINT Z (1 1 1)"
    assert point_z.to_wkt(dimension=3, use_old_3d=True) == "POINT (1 1 1)"


def test_to_wkb():
    be = b'\x00'
    le = b'\x01'
    point_type = b'\x01\x00\x00\x00'  # 1 as 32-bit uint (LE)
    point_type_3d = b'\x01\x00\x00\x80'
    coord = b'\x00\x00\x00\x00\x00\x00\xf0?'  # 1.0 as double (LE)

    assert point_z.to_wkb(dimension=2) == le + point_type + 2 * coord
    assert point_z.to_wkb(dimension=3) == le + point_type_3d + 3 * coord
    assert point_z.to_wkb(dimension=2, byte_order=0) == \
        be + point_type[::-1] + 2 * coord[::-1]


def test_to_wkb_with_srid():
    point_with_srid = pygeos.set_srid(point, np.int32(4326))
    result = point_with_srid.to_wkb(include_srid=True)
    assert np.frombuffer(result[5:9], '<u4').item() == 4326


def test_to_wkb_hex():
    le = b'01'
    point_type = b'01000000'
    coord = b'000000000000F03F'  # 1.0 as double (LE)

    assert point_z.to_wkb(hex=True, dimension=2) == le + point_type + 2 * coord


@pytest.mark.parametrize("geom", all_types)
def test_from_wkt(geom):
    wkt = geom.to_wkt()
    actual = pygeos.GEOSGeometry.from_wkt(wkt)
    assert pygeos.equals(actual, geom)


def test_from_wkt_bytes():
    actual = pygeos.GEOSGeometry.from_wkt(b'POINT (2 2)')
    assert pygeos.equals(actual, point)


def test_from_wkt_exceptions():
    with pytest.raises(TypeError):
        pygeos.GEOSGeometry.from_wkt(list("POINT (2 2)"))
    with pytest.raises(TypeError):
        pygeos.GEOSGeometry.from_wkt(None)
    with pytest.raises(pygeos.GEOSException):
        pygeos.GEOSGeometry.from_wkt('')
    with pytest.raises(pygeos.GEOSException):
        pygeos.GEOSGeometry.from_wkt('NOT A WKT STRING')


@pytest.mark.parametrize("geom", all_types)
@pytest.mark.parametrize("use_hex", [False, True])
@pytest.mark.parametrize("byte_order", [0, 1])
def test_from_wkb(geom, use_hex, byte_order):
    wkb = geom.to_wkb(hex=use_hex, byte_order=byte_order)
    actual = pygeos.GEOSGeometry.from_wkb(wkb)
    assert pygeos.equals(actual, geom)


def test_from_wkb_typeerror():
    with pytest.raises(TypeError):
        pygeos.GEOSGeometry.from_wkb("\x01")
    with pytest.raises(TypeError):
        pygeos.GEOSGeometry.from_wkb(None)
    with pytest.raises(pygeos.GEOSException):
        pygeos.GEOSGeometry.from_wkb(b'POINT (2 2)')



GEOS_POINT = 0
GEOS_LINESTRING = 1
GEOS_LINEARRING = 2
GEOS_POLYGON = 3
GEOS_MULTIPOINT = 4
GEOS_MULTILINESTRING = 5
GEOS_MULTIPOLYGON = 6
GEOS_GEOMETRYCOLLECTION = 7


from .ufuncs import GEOSGeometry  # NOQA
from .ufuncs import GEOSException  # NOQA
from .ufuncs import is_empty  # NOQA
from .ufuncs import is_simple  # NOQA
from .ufuncs import is_ring  # NOQA
from .ufuncs import has_z  # NOQA
from .ufuncs import is_closed  # NOQA
from .ufuncs import is_valid  # NOQA
from .ufuncs import disjoint  # NOQA
from .ufuncs import touches  # NOQA
from .ufuncs import intersects  # NOQA
from .ufuncs import crosses  # NOQA
from .ufuncs import within  # NOQA
from .ufuncs import contains  # NOQA
from .ufuncs import overlaps  # NOQA
from .ufuncs import equals  # NOQA
from .ufuncs import covers  # NOQA
from .ufuncs import covered_by  # NOQA
from .ufuncs import clone  # NOQA
from .ufuncs import envelope  # NOQA
from .ufuncs import convex_hull  # NOQA
from .ufuncs import boundary  # NOQA
from .ufuncs import unary_union  # NOQA
from .ufuncs import point_on_surface  # NOQA
from .ufuncs import get_centroid  # NOQA
from .ufuncs import line_merge  # NOQA
from .ufuncs import extract_unique_points  # NOQA
from .ufuncs import get_start_point  # NOQA
from .ufuncs import get_end_point  # NOQA
from .ufuncs import get_exterior_ring  # NOQA
from .ufuncs import normalize  # NOQA
from .ufuncs import get_interior_ring_n  # NOQA
from .ufuncs import get_point_n  # NOQA
from .ufuncs import get_geometry_n  # NOQA
from .ufuncs import set_srid  # NOQA
from .ufuncs import interpolate  # NOQA
from .ufuncs import interpolate_normalized  # NOQA
from .ufuncs import simplify  # NOQA
from .ufuncs import topology_preserve_simplify  # NOQA
from .ufuncs import intersection  # NOQA
from .ufuncs import difference  # NOQA
from .ufuncs import symmetric_difference  # NOQA
from .ufuncs import union  # NOQA
from .ufuncs import shared_paths  # NOQA
from .ufuncs import get_x  # NOQA
from .ufuncs import get_y  # NOQA
from .ufuncs import area  # NOQA
from .ufuncs import length  # NOQA
from .ufuncs import get_length  # NOQA
from .ufuncs import geom_type_id  # NOQA
from .ufuncs import get_dimensions  # NOQA
from .ufuncs import get_coordinate_dimensions  # NOQA
from .ufuncs import get_srid  # NOQA
from .ufuncs import get_num_geometries  # NOQA
from .ufuncs import get_num_interior_rings  # NOQA
from .ufuncs import get_num_points  # NOQA
from .ufuncs import get_num_coordinates  # NOQA
from .ufuncs import distance  # NOQA
from .ufuncs import hausdorff_distance  # NOQA
from .ufuncs import project  # NOQA
from .ufuncs import project_normalized  # NOQA
from .ufuncs import buffer  # NOQA
from .ufuncs import snap  # NOQA
from .ufuncs import equals_exact  # NOQA
from .construction import points, linestrings, linearrings, polygons, box, \
    multipoints, multilinestrings, multipolygons, geometrycollections  # NOQA



import numpy as np
from . import (ufuncs, GEOS_MULTIPOINT, GEOS_MULTIPOLYGON,
               GEOS_MULTILINESTRING, GEOS_GEOMETRYCOLLECTION)


def _wrap_construct_ufunc(func, coords, y=None, z=None):
    if y is None:
        return func(coords)
    x = coords
    if z is None:
        coords = np.broadcast_arrays(x, y)
    else:
        coords = np.broadcast_arrays(x, y, z)
    return func(np.stack(coords, axis=-1))


def points(coords, y=None, z=None):
    """Create an array of points.

    Attributes
    ----------
    coords : array_like
        An array of coordinate tuples (2- or 3-dimensional) or, if `y` is
        provided, an array of x coordinates.
    y : array_like
    z : array_like
    """
    return _wrap_construct_ufunc(ufuncs.points, coords, y, z)


def linestrings(coords, y=None, z=None):
    """Create an array of linestrings.

    Attributes
    ----------
    coords : array_like
        An array of lists of coordinate tuples (2- or 3-dimensional) or, if `y`
        is provided, an array of lists of x coordinates
    y : array_like
    z : array_like
    """
    return _wrap_construct_ufunc(ufuncs.linestrings, coords, y, z)


def linearrings(coords, y=None, z=None):
    """Create an array of linearrings.

    If the provided coords do not constitute a closed linestring, the first
    coordinate is duplicated at the end to close the ring.

    Attributes
    ----------
    coords : array_like
        An array of lists of coordinate tuples (2- or 3-dimensional) or, if `y`
        is provided, an array of lists of x coordinates
    y : array_like
    z : array_like
    """
    return _wrap_construct_ufunc(ufuncs.linearrings, coords, y, z)


def polygons(shells, holes=None):
    """Create an array of polygons.

    Attributes
    ----------
    shell : array_like
        An array of linearrings that constitute the out shell of the polygons.
        Coordinates can also be passed, see linearrings.
    holes : array_like
        An array of lists of linearrings that constitute holes for each shell.
    """
    shells = np.asarray(shells)
    if not isinstance(shells, ufuncs.GEOSGeometry) and \
            np.issubdtype(shells.dtype, np.number):
        shells = linearrings(shells)

    if holes is None:
        return ufuncs.polygons_without_holes(shells)

    holes = np.asarray(holes)
    if not isinstance(holes, ufuncs.GEOSGeometry) and \
            np.issubdtype(holes.dtype, np.number):
        holes = linearrings(holes)
    return ufuncs.polygons_with_holes(shells, holes)


def box(x1, y1, x2, y2):
    """Create box polygons.

    Attributes
    ----------
    x1 : array_like
    y2 : array_like
    x1 : array_like
    y2 : array_like
    """
    x1, y1, x2, y2 = np.broadcast_arrays(x1, y1, x2, y2)
    rings = np.array(((x2, y1), (x2, y2), (x1, y2), (x1, y1)))
    rings = np.moveaxis(rings, (0, 1), (-2, -1))
    return polygons(rings)


def multipoints(geometries):
    """Create multipoints from arrays of points

    Attributes
    ----------
    geometries : array_like
        An array of points or coordinates (see points).
    """
    geometries = np.asarray(geometries)
    if not isinstance(geometries, ufuncs.GEOSGeometry) and \
            np.issubdtype(geometries.dtype, np.number):
        geometries = points(geometries)
    return ufuncs.create_collection(geometries, GEOS_MULTIPOINT)


def multilinestrings(geometries):
    """Create multilinestrings from arrays of linestrings

    Attributes
    ----------
    geometries : array_like
        An array of linestrings or coordinates (see linestrings).
    """
    geometries = np.asarray(geometries)
    if not isinstance(geometries, ufuncs.GEOSGeometry) and \
            np.issubdtype(geometries.dtype, np.number):
        geometries = linestrings(geometries)
    return ufuncs.create_collection(geometries, GEOS_MULTILINESTRING)


def multipolygons(geometries):
    """Create multipolygons from arrays of polygons

    Attributes
    ----------
    geometries : array_like
        An array of polygons or coordinates (see polygons).
    """
    geometries = np.asarray(geometries)
    if not isinstance(geometries, ufuncs.GEOSGeometry) and \
            np.issubdtype(geometries.dtype, np.number):
        geometries = polygons(geometries)
    return ufuncs.create_collection(geometries, GEOS_MULTIPOLYGON)


def geometrycollections(geometries):
    """Create geometrycollections from arrays of geometries

    Attributes
    ----------
    geometries : array_like
        An array of geometries
    """
    return ufuncs.create_collection(geometries, GEOS_GEOMETRYCOLLECTION)



import os
from setuptools import setup, Extension
import geosconfig

module_ufuncs = Extension(
    'pygeos.ufuncs',
    sources=['src/ufuncs.c'],
    include_dirs=geosconfig.include_dirs,
    library_dirs=geosconfig.library_dirs,
    libraries=geosconfig.libraries,
    extra_link_args=geosconfig.extra_link_args
)

try:
    descr = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()
except IOError:
    descr = ''


setup(
    name='pygeos',
    version='0.3',
    description='GEOS wrapped in numpy ufuncs',
    long_description=descr,
    url='https://github.com/caspervdw/pygeos',
    author='Casper van der Wel',
    license='BSD 3-Clause',
    packages=['pygeos'],
    install_requires=['numpy'],
    test_requires=['pytest'],
    python_requires='>=3',
    include_package_data=True,
    ext_modules=[module_ufuncs],
    classifiers=[
        'Programming Language :: Python :: 3',
        'Development Status :: 1 - Planning',
        'Topic :: Scientific/Engineering :: Mathematics',
        'Topic :: Scientific/Engineering :: GIS',
        'Operating System :: Unix',
    ]
)
